#!/usr/local/bin/perl

#
# Counts votes from text files
#
# usage:
# votecount.pl file1 file2 ...

$progname = "votecount.pl";
$argc = scalar(@ARGV);

$startitem = 114;
$maxvotes = 144;
$voters=0;

# For each file specified on the command line
foreach $vfilename (@ARGV) {

	$item = $startitem;	#starting item number
	$votecount = 0;
	$voters++;

	#print "$vfilename\n";

	open(vfile, $vfilename) or die "Can't open file $vfilename: $!\n";

	while (<vfile>) {
		if (/^$item/) { 
			@fields = split;
			#print "$vfilename @fields[1]\n";

			#if ($item == $startitem) { print "X$fields[1]X\n"; }

			if ($fields[1] =~ /\(A\)/) {
				@accept[$item]++;
			}

			if ($fields[1] =~ /\(R\)/) {
				@reject[$item]++;
			}

			if ($fields[1] =~ /\(M\)/) {
				@modify[$item]++;
			}

			if ($fields[1] =~ /\(B\)/) {
				@abstain[$item]++;
			}

			$item++;
			$votecount++;
		}
	}

	if ($votecount != $maxvotes) {
		print "Error: file $vfilename has invalid vote count; counted $votecount\n";
	}

	close(vfile);
}

$techvotes = int $voters * .75;

open(ctfile, "commenttypes.txt") or die "Can't open file commenttypes.txt: $!\n";

while (<ctfile>) {
	@fields = split;
	if (/echnical/) {
		@technical[@fields[0]] = 1;
	} else {
		@technical[@fields[0]] = 0;
	}

	#printf "%d: %d\n", @fields[0], @technical[@fields[0]];
}

close(ctfile);


print "Item# Type Acc  Rej Mod aBs Result\n";
$item = $startitem;
while ($item < $startitem + $maxvotes) {
	$result[$item] = "F";

	if (@technical[$item]) {
		$t = "T";
	} else {
		$t = "E";
	}
	printf "%3d: %4s %3d %3d %3d %3d", $item, $t, @accept[$item], @reject[$item], @modify[$item], @abstain[$item];

	if (@accept[$item] + 
		@reject[$item] +
		@modify[$item] +
		@abstain[$item] != $voters) {

		print "***Vote miscount on item $item\n";
		$item++;
		next;
	}


	#if accept wins
	if (@accept[$item] > @reject[$item] &&
		 @accept[$item] > @modify[$item] &&
		 @accept[$item] > @abstain[$item]) {

		$resultstr = "   Accepted by ";
		if (@accept[$item] >= $techvotes) {
			$resultstr .= ">= 75\% majority.";
			$result[$item] = "A";
		} else {
			if (@technical[$item]) {
				$margin = $techvotes - @accept[$item];
				$resultstr = "   Failed to achieve 75\% accept votes by $margin.";
				$result[$item] = "FA";
			} elsif (@accept[$item] > int (.50 * $voters)) {
				$resultstr .= ">= 50\% majority.";
				$result[$item] = "A";
			} else {
				$resultstr .= "simple majority.";
				$result[$item] = "A";
			}
		}

		print "$resultstr\n";
	}

	#if reject wins
	elsif (@reject[$item] > @accept[$item] &&
		 @reject[$item] > @modify[$item] &&
		 @reject[$item] > @abstain[$item]) {

		$resultstr = "   Rejected by ";
		if (@reject[$item] >= $techvotes) {
			$resultstr .= ">= 75\% majority.";
			$result[$item] = "R";
		} else {
			if (@reject[$item] > int (.50 * $voters)) {
				$resultstr .= ">= 50\% majority.";
				$result[$item] = "R";
			} else {
				$resultstr .= "simple majority.";
				$result[$item] = "R";
			}
		}

		print "$resultstr\n";
	}

	#if modify wins
	elsif (@modify[$item] > @reject[$item] &&
		 @modify[$item] > @accept[$item] &&
		 @modify[$item] > @abstain[$item]) {

		$resultstr =  "   Accepted, but only if modified by ";
		if (@modify[$item] >= $techvotes) {
			$resultstr .= ">= 75\% majority.";
			$result[$item] = "M";
		} else {
			if (@modify[$item] > int (.50 * $voters)) {
				$resultstr .= ">= 50\% majority.";
				$result[$item] = "M";
			} else {
				$resultstr .= "simple majority.";
				$result[$item] = "M";
			}
		}

		print "$resultstr\n";
	}

	#if abstain wins
	elsif (@abstain[$item] > @reject[$item] &&
		 @abstain[$item] > @modify[$item] &&
		 @abstain[$item] > @accept[$item]) {

		$resultstr = "   Abstains win by ";
		if (@abstain[$item] >= $techvotes) {
			$resultstr .= ">= 75\% majority.";
			$result[$item] = "B";
		} else {
			if (@abstain[$item] > int (.50 * $voters)) {
				$resultstr .= ">= 50\% majority.";
				$result[$item] = "B";
			} else {
				$resultstr .= "simple majority.";
				$result[$item] = "B";
			}
		}

		print "$resultstr\n";
	}

	else {
		if (@technical[$item]) {
			print "   Failed to achieve majority (tie).\n";
		} else {
			print "   *** Tie vote ***.\n";
		}
	}

	$item ++;
}

#
# Read commentor names
#
open(nfile, "number_name.txt") or die "Can't open file commenttypes.txt: $!\n";

while (<nfile>) {
	@fields = split;
	$x = substr($_, index($_, @fields[1]));
	@cname[@fields[0]] = $x;
	#print "$x";
}

close(nfile);

#
# Extract reasons from failed comments.  Indicate voter name and vote
#

#
# First, find all comments that might have passed, but failed to get 75%
# majority and print the M votes.
#
print "\n***Reason fields for comments that failed to achieve 75% majority follow:***\n\n";

foreach $vfilename (@ARGV) {

	$item = $startitem;	#starting item number

	open(vfile, $vfilename) or die "Can't open file $vfilename: $!\n";

	$state = 0;

	while (<vfile>) {
		@fields = split;

		if (/Last/) {
			$lastname = @fields[3];
			next;
		}

		if (/First/) {
			$firstname = @fields[3];
			next;
		}

		if (/^$item/) { 

			$state = 0;

			# if this item failed to achieve a majority
			if ($result[$item] eq "FA") {
				$v = substr $fields[1], 0, 3;

				if ($v eq "(M)") {
					$state = 1;
					$reason = "   $firstname $lastname voted $v because:\n";

					$line = substr($_, (index $_, @fields[1]) + 3);
					$line =~ s/^\s*//; #strip leading white space
					$reason .= "      $line";

					@reasons[$item] .= $reason;
				}
			}

			$item++;
			next;
		}

		#if processing a multiline description

		if ($state == 1) {
			s/^\s*//; #strip leading white space
			
			# if it's a blank line, don't save it
			if ($_ ne "") {
				@reasons[$item-1] .= "      $_";
			}
		}
	}
}

#
# Next, print the R votes.
#
foreach $vfilename (@ARGV) {

	$item = $startitem;	#starting item number

	open(vfile, $vfilename) or die "Can't open file $vfilename: $!\n";

	$state = 0;

	while (<vfile>) {
		@fields = split;

		if (/Last/) {
			$lastname = @fields[3];
			next;
		}

		if (/First/) {
			$firstname = @fields[3];
			next;
		}

		if (/^$item/) { 

			$state = 0;

			# if this item failed to achieve a majority
			if ($result[$item] eq "FA") {
				$v = substr $fields[1], 0, 3;

				if ($v eq "(R)") {
					$state = 1;
					$reason = "   $firstname $lastname voted $v because:\n";

					$line = substr($_, (index $_, @fields[1]) + 3);
					$line =~ s/^\s*//; #strip leading white space
					$reason .= "      $line";

					@reasons[$item] .= $reason;
				}
			}

			$item++;
			next;
		}

		#if processing a multiline description

		if ($state == 1) {
			s/^\s*//; #strip leading white space
			
			# if it's a blank line, don't save it
			if ($_ ne "") {
				@reasons[$item-1] .= "      $_";
			}
		}
	}
}

#
# Next, print the B votes.
#
foreach $vfilename (@ARGV) {

	$item = $startitem;	#starting item number

	open(vfile, $vfilename) or die "Can't open file $vfilename: $!\n";

	$state = 0;

	while (<vfile>) {
		@fields = split;

		if (/Last/) {
			$lastname = @fields[3];
			next;
		}

		if (/First/) {
			$firstname = @fields[3];
			next;
		}

		if (/^$item/) { 

			$state = 0;

			# if this item failed to achieve a majority
			if ($result[$item] eq "FA") {
				$v = substr $fields[1], 0, 3;

				if ($v eq "(B)") {
					$reason = "   $firstname $lastname voted $v because:\n";

					$line = substr($_, (index $_, @fields[1]) + 3);
					$line =~ s/^\s*//; #strip leading white space


					# check for empty reason
					if ($line ne "") {
						$state = 1;
						$reason .= "      $line";
					} else {
						$reason .= "      No reason given.\n";
					}

					@reasons[$item] .= "$reason";
				}
			}

			$item++;
			next;
		}

		#if processing a multiline description

		if ($state == 1) {
			s/^\s*//; #strip leading white space
			
			# if it's a blank line, don't save it
			if ($_ ne "") {
				@reasons[$item-1] .= "      $_";
			}
		}
	}
}

#foreach $reason (@reasons) {
#	print $reason;
#}

$item = $startitem;
while ($item < $startitem + $maxvotes) {
	if ($result[$item] eq "FA") {
		print "$item: author=@cname[$item]";
		print @reasons[$item];
	}
	$item++;
}

# clear out the reasons buffer
@reasons = 0;

#
# Now, display the reason fields for any vote
#
print "\n***Reason fields for all comments follow:***\n\n";
foreach $vfilename (@ARGV) {

	$item = $startitem;	#starting item number

	open(vfile, $vfilename) or die "Can't open file $vfilename: $!\n";

	$state = 0;

	while (<vfile>) {
		@fields = split;

		if (/Last/) {
			$lastname = @fields[3];
			next;
		}

		if (/First/) {
			$firstname = @fields[3];
			next;
		}

		if (/^$item/) { 

			$state = 0;

			$v = substr $fields[1], 0, 3;

			$reason = "   $firstname $lastname voted $v because:\n";

			$line = substr($_, (index $_, @fields[1]) + 3);
			$line =~ s/^\s*//; #strip leading white space
			$reason .= "      $line";

			# check for empty reason
			if ($line ne "") {
				$state = 1;
				@reasons[$item] .= "$reason";
			}

			$item++;
			next;
		}

		#if processing a multiline description

		if ($state == 1) {
			s/^\s*//; #strip leading white space
			
			# if it's a blank line, don't save it
			if ($_ ne "") {
				@reasons[$item-1] .= "      $_";
			}
		}
	}
}

#foreach $reason (@reasons) {
#	print $reason;
#}

$item = $startitem;
while ($item < $startitem + $maxvotes) {
	print "$item: author=@cname[$item]";
	print @reasons[$item];
	$item++;
}

